"use strict";
const {CoreDefine, CoreClass} = require('@cutos/core');
const {SerialPort} = require("@cutos/drivers");
const {DeviceHeightWeightScaleSimulator} = require("./simulator/device-height-weight-scale-simulator");
const {TYPE, CMD,} = require('./device-height-weight-scale-def');
const {request} = require('./device-height-weight-scale-protocol')
const {opts} = require('./config.json');

class DeviceHeightWeightScale extends CoreClass.Driver {
    constructor(args) {
        super(args.name, TYPE);
        if (args.name === 'mock') {
            this.simulator = new DeviceHeightWeightScaleSimulator()
        }
        this.startBeat();
        this.updateStatusInfo(CoreDefine.HEARTBEAT_STATUS.ALIVE, "alive");
        this.onCommand(({cmd, args}, callback) => {
            console.log(cmd)
            switch (cmd) {
                // 必须实现
                case CMD.CONNECT:
                    this.connect(args, callback);
                    break;
                case CMD.SWITCH_ON:
                    this.switchOn(args, callback);
                    break;
                case CMD.SWITCH_OFF:
                    this.switchOff(args, callback);
                    break;
                case CMD.GET_HEIGHT_WEIGHT:
                    this.getHeightWeight(args, callback);
                    break;
                default:
                    this.unsupported(cmd, args, callback);
                    break;
            }
        });
    }

    unsupported(cmd, args, callback) {
        let result = {};
        result.msg = "cmd: " + cmd + " is unsupported.";
        result.status = false;
        callback(result);
    }

    connect(args, callback) {
        let params = {...opts, ...args}
        if (this.port) {
            callback({status: true, msg: params})
            return
        }
        this.port = new SerialPort(params.path, params.baudRate, err => {
            if (err) {
                callback({status: false, msg: err.message})
            } else {
                callback({status: true, msg: params})
            }
        }, this.simulator)

        this.port.on('data', data => this.onSerialData(data))
    }

    onSerialData(data) {
        if (data[1] === 0x08 && data[2] === 0x04) {
            this.onHeightWeightData(data)
        }

        if (data[1] === 0x06 && data[2] === 0x03) {
            this.onWeightData(data);
        }

        if (data[1] === 0x06 && data[2] === 0x01) {
            this.onSwitchOnData(data)
        }

        if (data[1] === 0x06 && data[2] === 0x09) {
            this.onSwitchOffData(data)
        }
    }

    onHeightWeightData(data) {
        let height = data[3] * 256 + data[4]
        let weight = data[5] * 256 + data[6]
        height = height / 10
        weight = weight / 10
        height = parseFloat(height.toFixed(1))
        weight = parseFloat(weight.toFixed(1))
        // if (this.heightWeightCallback) {
        //     this.heightWeightCallback({status: true, msg: {height, weight}})
        //     this.heightWeightCallback = null
        // }
        this.sendData({
            type: 'height-weight',
            values: {height, weight}
        })
    }

    onWeightData(data) {
        let weight = data[3] * 256 + data[4]
        weight = weight / 10
        weight = parseFloat(weight.toFixed(1))
        this.sendData({
            type: 'weight',
            values: {weight}
        })
    }

    onSwitchOnData(data) {
        // if (this.switchOnCallback) {
        //     this.switchOnCallback({status: true})
        //     this.switchOnCallback = null
        //     this.isReady = true
        //     clearInterval(this.timer)
        // }
        this.isReady = true
        clearInterval(this.timer)
        this.sendData({
            type: 'switch-on',
            values: {status: true}
        })
    }

    onSwitchOffData(data) {
        // if (this.switchOffCallback) {
        //     this.switchOffCallback({status: true})
        //     this.switchOffCallback = null
        // }
        this.sendData({
            type: 'switch-off',
            values: {status: true}
        })
    }

    getHeightWeight(args, callback) {
        // this.heightWeightCallback = callback
        this.port.write(request.GET_HEIGHT_WEIGHT)
    }

    switchOn(args, callback) {
        // this.switchOnCallback = callback
        this.port.write(request.SWITCH_ON)
        if (!this.isReady) {
            this.timer = setInterval(() => this.port.write(request.SWITCH_ON), 500)
        }
    }

    switchOff(args, callback) {
        // this.switchOffCallback = callback
        this.port.write(request.SWITCH_OFF)
    }
}

module.exports = DeviceHeightWeightScale;
